@@ -62,6 +62,37 @@ const UnicodeString &LexdCompiler::name(string_ref r) const
   return id_to_name[(unsigned int)r];
 }
 
+UnicodeString LexdCompiler::printPattern(const pattern_element_t& pat)
+{
+  UnicodeString ret = name(pat.left.name);
+  auto& pos = pat.tag_filter.pos();
+  auto& neg = pat.tag_filter.neg();
+  if (!pos.empty() || !neg.empty()) {
+    ret += '[';
+    int ln = ret.length();
+    for (auto& it : pos) {
+      if (ret.length() > ln) ret += ',';
+      ret += name(it);
+    }
+    for (auto& it : neg) {
+      if (ret.length() > ln) ret += ',';
+      ret += '-';
+      ret += name(it);
+    }
+    ret += ']';
+  }
+  switch (pat.mode) {
+  case Question:
+    ret += '?'; break;
+  case Plus:
+    ret += '+'; break;
+  case Star:
+    ret += '*'; break;
+  default: break;
+  }
+  return ret;
+}
+
 trans_sym_t LexdCompiler::alphabet_lookup(const UnicodeString &symbol)
 {
   if (!symbol.hasMoreChar32Than(0, symbol.length(), 1)) {
@@ -79,24 +110,20 @@ trans_sym_t LexdCompiler::alphabet_lookup(trans_sym_t l, trans_sym_t r)
 }
 
 LexdCompiler::LexdCompiler()
-  : shouldAlign(false), shouldCompress(false), shouldCombine(true),
-    tagsAsFlags(false), shouldHypermin(false), tagsAsMinFlags(false),
-    input(NULL), inLex(false), inPat(false), lineNumber(0), doneReading(false),
-    anonymousCount(0), transitionCount(0)
 {
   id_to_name.push_back("");
   name_to_id[""] = string_ref(0);
   lexicons[string_ref(0)] = vector<entry_t>();
 
   left_sieve_name = internName("<");
-  token_t lsieve_tok = {.name=left_sieve_name, .part=1};
+  token_t lsieve_tok = {.name=left_sieve_name, .part=1, .optional=false};
   pattern_element_t lsieve_elem = {.left=lsieve_tok, .right=lsieve_tok,
                                    .tag_filter=tag_filter_t(),
                                    .mode=Normal};
   left_sieve_tok = vector<pattern_element_t>(1, lsieve_elem);
 
   right_sieve_name = internName(">");
-  token_t rsieve_tok = {.name=right_sieve_name, .part=1};
+  token_t rsieve_tok = {.name=right_sieve_name, .part=1, .optional=false};
   pattern_element_t rsieve_elem = {.left=rsieve_tok, .right=rsieve_tok,
                                    .tag_filter=tag_filter_t(),
                                    .mode=Normal};
@@ -106,13 +133,10 @@ LexdCompiler::LexdCompiler()
 LexdCompiler::~LexdCompiler()
 {}
 
-const UChar*
-err(const UnicodeString& s)
-{
-  // u_*printf only accept const UChar*
-  // so here's a wrapper so we don't have to write all this out every time
-  return to_ustring(s).c_str();
-}
+// u_*printf only accept const UChar*
+// so here's a wrapper so we don't have to write all this out every time
+// and make it a macro so we don't have issues with it deallocating
+#define err(s) (to_ustring(s).c_str())
 
 void
 LexdCompiler::die(const char* msg, ...)
@@ -255,6 +279,212 @@ LexdCompiler::readTagFilter(char_iter& iter, UnicodeString& line)
   return tag_filter_t();
 }
 
+void
+LexdCompiler::readSymbol(char_iter& iter, UnicodeString& line, lex_token_t& tok)
+{
+  if (*iter == "\\") {
+    if (shouldCombine) {
+      tok.symbols.push_back(alphabet_lookup(*++iter));
+    } else {
+      ++iter;
+      for (int c = 0; c < (*iter).length(); c++) {
+        tok.symbols.push_back(alphabet_lookup((*iter)[c]));
+      }
+    }
+  } else if (*iter == "{" || *iter == "<") {
+    UChar end = (*iter == "{") ? '}' : '>';
+    int i = iter.span().first;
+    for (; iter != iter.end() && *iter != end; ++iter) ;
+
+    if (*iter == end) {
+      tok.symbols.push_back(alphabet_lookup(line.tempSubStringBetween(i, iter.span().second)));
+    } else {
+      die("Multichar symbol didn't end; searching for %S", err(end));
+    }
+  } else if (shouldCombine) {
+    tok.symbols.push_back(alphabet_lookup(*iter));
+  } else {
+    for (int c = 0; c < (*iter).length(); c++) {
+      tok.symbols.push_back(alphabet_lookup((*iter)[c]));
+    }
+  }
+}
+
+int
+LexdCompiler::processRegexTokenSeq(char_iter& iter, UnicodeString& line, Transducer* trans, int start_state)
+{
+  bool inleft = true;
+  vector<vector<lex_token_t>> left, right;
+  for (; iter != iter.end(); ++iter) {
+    if (*iter == "(" || *iter == ")" || *iter == "|" || *iter == "/") break;
+    else if (*iter == "?" || *iter == "*" || *iter == "+")
+      die("Quantifier %S may only be applied to parenthesized groups", err(*iter));
+    else if (*iter == "]") die("Regex contains mismatched ]");
+    else if (*iter == ":" && inleft) inleft = false;
+    else if (*iter == ":") die("Regex contains multiple colons");
+    else if (*iter == "[") {
+      ++iter;
+      vector<lex_token_t> sym;
+      for (; iter != iter.end(); ++iter) {
+        if (*iter == "]") break;
+        else if (*iter == "-" && !sym.empty()) {
+          ++iter;
+          if (*iter == "]" || iter == iter.end()) {
+            --iter;
+            lex_token_t temp;
+            readSymbol(iter, line, temp);
+            sym.push_back(temp);
+          } else {
+            lex_token_t start = sym.back();
+            lex_token_t end;
+            readSymbol(iter, line, end);
+            // This will fail on diacritics even with -U
+            // on the principle that command-line args should not
+            // change the validity of the code -DGS 2022-05-17
+            if (start.symbols.size() != 1 || end.symbols.size() != 1 ||
+                (int)start.symbols[0] <= 0 || (int)end.symbols[0] <= 0)
+              die("Cannot process symbol range between multichar symbols");
+            int i_start = (int)start.symbols[0];
+            int i_end = (int)end.symbols[0];
+            if (i_start > i_end)
+              die("First character in symbol range does not preceed last");
+            for (int i = 1 + i_start; i <= i_end; i++) {
+              lex_token_t mid;
+              mid.symbols.push_back((trans_sym_t)i);
+              sym.push_back(mid);
+            }
+          }
+        } else {
+          lex_token_t temp;
+          readSymbol(iter, line, temp);
+          sym.push_back(temp);
+        }
+      }
+      (inleft ? left : right).push_back(sym);
+    } else {
+      vector<lex_token_t> v_temp;
+      lex_token_t t_temp;
+      readSymbol(iter, line, t_temp);
+      v_temp.push_back(t_temp);
+      (inleft ? left : right).push_back(v_temp);
+    }
+  }
+  int state = start_state;
+  vector<lex_token_t> empty_vec;
+  lex_token_t empty_tok;
+  empty_tok.symbols.push_back(trans_sym_t());
+  empty_vec.push_back(empty_tok);
+  for (unsigned int i = 0; i < left.size() || i < right.size(); i++) {
+    vector<lex_token_t>& lv = (i < left.size() && !left[i].empty() ?
+                               left[i] : empty_vec);
+    vector<lex_token_t>& rv = (i < right.size() && !right[i].empty() ?
+                               right[i] : empty_vec);
+    bool first = true;
+    int dest_state = 0;
+    if (inleft) {
+      for (auto& s : lv) {
+        if (first) {
+          dest_state = state;
+          for (auto& it : s.symbols)
+            dest_state = trans->insertNewSingleTransduction(alphabet((int)it, (int)it), dest_state);
+          if (dest_state == state)
+            dest_state = trans->insertNewSingleTransduction(0, dest_state);
+          first = false;
+        } else if (s.symbols.empty()) {
+          trans->linkStates(state, dest_state, 0);
+        } else {
+          int cur_state = state;
+          for (unsigned int k = 0; k < s.symbols.size(); k++) {
+            if (k+1 == s.symbols.size())
+              trans->linkStates(cur_state, dest_state, alphabet((int)s.symbols[k], (int)s.symbols[k]));
+            else
+              cur_state = trans->insertNewSingleTransduction(alphabet((int)s.symbols[k], (int)s.symbols[k]), cur_state);
+          }
+        }
+      }
+    } else {
+      for (auto& l : lv) {
+        for (auto& r : rv) {
+          vector<int> paired;
+          for (unsigned int j = 0; j < l.symbols.size() || j < r.symbols.size(); j++) {
+            trans_sym_t ls = (j < l.symbols.size() ? l.symbols[j] : trans_sym_t());
+            trans_sym_t rs = (j < r.symbols.size() ? r.symbols[j] : trans_sym_t());
+            paired.push_back(alphabet((int)ls, (int)rs));
+          }
+          if (first) {
+            dest_state = state;
+            for (auto& it : paired) {
+              dest_state = trans->insertNewSingleTransduction(it, dest_state);
+            }
+            first = false;
+          } else {
+            int cur_state = state;
+            for (unsigned int k = 0; k < paired.size(); k++) {
+              if (k+1 == paired.size())
+                trans->linkStates(cur_state, dest_state, paired[k]);
+              else
+                cur_state = trans->insertNewSingleTransduction(paired[k], cur_state);
+            }
+          }
+        }
+      }
+    }
+    state = dest_state;
+  }
+  return state;
+}
+
+int
+LexdCompiler::processRegexGroup(char_iter& iter, UnicodeString& line, Transducer* trans, int start_state, unsigned int depth)
+{
+  ++iter; // initial slash or paren
+  int state = start_state;
+  vector<int> option_ends;
+  for (; iter != iter.end(); ++iter) {
+    if (*iter == "(") {
+      state = trans->insertNewSingleTransduction(0, state);
+      state = processRegexGroup(iter, line, trans, state, depth+1);
+      --iter;
+      // this function ends on character after close paren or quantifier
+      // so step back so loop increment doesn't skip a character
+    }
+    else if (*iter == ")" || *iter == "/") break;
+    else if (*iter == "|") {
+      if (state == start_state)
+        state = trans->insertNewSingleTransduction(0, state);
+      option_ends.push_back(state);
+      state = start_state;
+    }
+    else {
+      state = processRegexTokenSeq(iter, line, trans, state);
+      --iter;
+    }
+  }
+  if (state == start_state)
+    state = trans->insertNewSingleTransduction(0, state);
+  for (auto& it : option_ends)
+    trans->linkStates(it, state, 0);
+  if ((depth > 0 && *iter == "/") || (depth == 0 && *iter == ")"))
+    die("Mismatched parentheses in regex");
+  if (iter == iter.end())
+    die("Unterminated regex");
+  ++iter;
+  if (depth > 0) {
+    if (*iter == "?") {
+      trans->linkStates(start_state, state, 0);
+      ++iter;
+    } else if (*iter == "*") {
+      trans->linkStates(start_state, state, 0);
+      trans->linkStates(state, start_state, 0);
+      ++iter;
+    } else if (*iter == "+") {
+      trans->linkStates(state, start_state, 0);
+      ++iter;
+    }
+  }
+  return state;
+}
+
 lex_seg_t
 LexdCompiler::processLexiconSegment(char_iter& iter, UnicodeString& line, unsigned int part_count)
 {
@@ -276,9 +506,11 @@ LexdCompiler::processLexiconSegment(char_iter& iter, UnicodeString& line, unsign
   }
   if((*iter).startsWith("/") && seg.left.symbols.size() == 0)
   {
-    die("Lexicon entries beginning with unescaped / are reserved for future use. Please use \\/");
+    seg.regex = new Transducer();
+    int state = processRegexGroup(iter, line, seg.regex, 0, 0);
+    seg.regex->setFinal(state);
   }
-  if(iter == iter.end() && seg.left.symbols.size() == 0)
+  if(iter == iter.end() && seg.regex == nullptr && seg.left.symbols.size() == 0)
     die("Expected %d parts, found %d", currentLexiconPartCount, part_count);
   for(; iter != iter.end(); ++iter)
   {
@@ -300,51 +532,17 @@ LexdCompiler::processLexiconSegment(char_iter& iter, UnicodeString& line, unsign
       else
         die("Lexicon entry contains multiple colons");
     }
-    else if(*iter == "\\")
-    {
-	  if(shouldCombine)
-	  {
-		(inleft ? seg.left : seg.right).symbols.push_back(alphabet_lookup(*++iter));
-	  }
-	  else
-	  {
-		++iter;
-		for(int c = 0; c < (*iter).length(); c++)
-		{
-		  (inleft ? seg.left : seg.right).symbols.push_back(alphabet_lookup((*iter)[c]));
-		}
-	  }
-    }
-    else if(*iter == "{" || *iter == "<")
-    {
-      UChar end = (*iter == "{") ? '}' : '>';
-      int i = iter.span().first;
-      for(; iter != iter.end() && *iter != end; ++iter) ;
-
-      if(*iter == end)
-      {
-        trans_sym_t sym = alphabet_lookup(line.tempSubStringBetween(i, iter.span().second));
-        (inleft ? seg.left : seg.right).symbols.push_back(sym);
-      }
-      else
-      {
-        die("Multichar entry didn't end; searching for %S", err(end));
-      }
-    }
-	else if(!shouldCombine)
-	{
-	  for(int c = 0; c < (*iter).length(); c++)
-	  {
-		(inleft ? seg.left : seg.right).symbols.push_back(alphabet_lookup((*iter)[c]));
-	  }
-	}
-    else (inleft ? seg.left : seg.right).symbols.push_back(alphabet_lookup(*iter));
+    else readSymbol(iter, line, (inleft ? seg.left : seg.right));
   }
   if(inleft)
   {
     seg.right = seg.left;
   }
 
+  if (seg.regex != nullptr &&
+      !(seg.left.symbols.empty() && seg.right.symbols.empty()))
+    die("Lexicon entry contains both regex and text");
+
   seg.tags = currentLexicon_tags;
 
   if(!tags.try_apply(seg.tags))
@@ -372,6 +570,16 @@ LexdCompiler::readToken(char_iter& iter, UnicodeString& line)
   if(name.length() == 0)
     die("Symbol '%S' without lexicon name at u16 %d-%d", err(*iter), iter.span().first, iter.span().second-1);
 
+  bool optional = false;
+  if(*iter == "?") {
+    iter++;
+    if(*iter == "(") {
+      optional = true;
+    } else {
+      iter--;
+    }
+  }
+
   unsigned int part = 1;
   if(*iter == "(")
   {
@@ -390,7 +598,7 @@ LexdCompiler::readToken(char_iter& iter, UnicodeString& line)
     ++iter;
   }
 
-  return token_t {.name = internName(name), .part = part};
+  return token_t {.name = internName(name), .part = part, .optional = optional};
 }
 
 RepeatMode
@@ -544,7 +752,7 @@ LexdCompiler::processPattern(char_iter& iter, UnicodeString& line)
       }
       ++iter;
       pattern_element_t anon;
-      anon.left = {.name=currentLexiconId, .part=1};
+      anon.left = {.name=currentLexiconId, .part=1, .optional=false};
       anon.right = anon.left;
       anon.mode = readModifier(iter);
       alternation.push_back(anon);
@@ -573,7 +781,7 @@ LexdCompiler::processPattern(char_iter& iter, UnicodeString& line)
         alternation.clear();
       }
       pattern_element_t anon;
-      anon.left = {.name=currentPatternId, .part=1};
+      anon.left = {.name=currentPatternId, .part=1, .optional=false};
       anon.right = anon.left;
       anon.mode = readModifier(iter);
       anon.tag_filter = filter;
@@ -847,6 +1055,7 @@ LexdCompiler::buildPattern(int state, Transducer* t, const pattern_t& pat, const
             matchedParts.find(tok.right.name) == matchedParts.end())
     {
       unsigned int max = lexicons[tok.left.name || tok.right.name].size();
+      if (tok.optional()) max++;
       for(unsigned int index = 0; index < max; index++)
       {
         Transducer *lex = getLexiconTransducer(pat[pos], index, false);
@@ -883,7 +1092,7 @@ LexdCompiler::buildPattern(int state, Transducer* t, const pattern_t& pat, const
   else
   {
     Transducer *p = buildPattern(tok);
-    if(p)
+    if(!p->hasNoFinals())
     {
       int new_state = t->insertTransducer(state, *p);
       if(tok.mode & Optional)
@@ -892,7 +1101,6 @@ LexdCompiler::buildPattern(int state, Transducer* t, const pattern_t& pat, const
         t->linkStates(new_state, state, 0);
       buildPattern(new_state, t, pat, is_free, pos+1);
     }
-    return;
   }
 }
 
@@ -900,11 +1108,24 @@ vector<int>
 LexdCompiler::determineFreedom(pattern_t& pat)
 {
   vector<int> is_free = vector<int>(pat.size(), 0);
+  map<string_ref, bool> is_optional;
   for(unsigned int i = 0; i < pat.size(); i++)
   {
+    const pattern_element_t& t1 = pat[i];
+    if (is_optional.find(t1.left.name) != is_optional.end() && is_optional[t1.left.name] != t1.optional()) {
+      die("Lexicon %S cannot be both optional and non-optional in a single pattern.", err(name(t1.left.name)));
+    }
+    if (is_optional.find(t1.right.name) != is_optional.end() && is_optional[t1.right.name] != t1.optional()) {
+      die("Lexicon %S cannot be both optional and non-optional in a single pattern.", err(name(t1.right.name)));
+    }
+    if (t1.left.name.valid()) {
+      is_optional[t1.left.name] = t1.optional();
+    }
+    if (t1.right.name.valid()) {
+      is_optional[t1.right.name] = t1.optional();
+    }
     if(is_free[i] != 0)
       continue;
-    const pattern_element_t& t1 = pat[i];
     for(unsigned int j = i+1; j < pat.size(); j++)
     {
       const pattern_element_t& t2 = pat[j];
@@ -927,6 +1148,7 @@ LexdCompiler::buildPattern(const pattern_element_t &tok)
     die("Cannot build collated pattern %S", err(name(tok.left.name)));
   if(patternTransducers.find(tok) == patternTransducers.end())
   {
+    if (verbose) cerr << "Compiling " << to_ustring(printPattern(tok)) << endl;
     Transducer* t = new Transducer();
     patternTransducers[tok] = NULL;
     map<string_ref, unsigned int> tempMatch;
@@ -953,13 +1175,21 @@ LexdCompiler::buildPattern(const pattern_element_t &tok)
     tempMatch.swap(matchedParts);
     if(!t->hasNoFinals())
     {
+      if (verbose)
+        cerr << "Minimizing " << to_ustring(printPattern(tok)) << endl;
       t->minimize();
-      patternTransducers[tok] = t;
     }
+    else if (verbose) {
+      cerr << "Warning: " << to_ustring(printPattern(tok));
+      cerr << " is empty." << endl;
+    }
+    patternTransducers[tok] = t;
+    if (verbose)
+      cerr << "Done with " << to_ustring(printPattern(tok)) << endl;
   }
   else if(patternTransducers[tok] == NULL)
   {
-    die("Cannot compile self-recursive or empty pattern '%S'", err(name(tok.left.name)));
+    die("Cannot compile self-recursive %S", err(printPattern(tok)));
   }
   return patternTransducers[tok];
 }
@@ -1018,6 +1248,7 @@ LexdCompiler::buildPatternWithFlags(const pattern_element_t &tok, int pattern_st
 {
   if(patternTransducers.find(tok) == patternTransducers.end())
   {
+    if (verbose) cerr << "Compiling " << to_ustring(printPattern(tok)) << endl;
     Transducer* trans = (shouldHypermin ? hyperminTrans : new Transducer());
     patternTransducers[tok] = NULL;
     unsigned int transition_index = 0;
@@ -1216,8 +1447,11 @@ LexdCompiler::buildPatternWithFlags(const pattern_element_t &tok, int pattern_st
       }
       else
       {
-        if(!trans->hasNoFinals())
+        if(!trans->hasNoFinals()) {
+          if (verbose)
+            cerr << "Minimizing " << to_ustring(printPattern(tok)) << endl;
           trans->minimize();
+        }
       }
     }
     else
@@ -1229,6 +1463,7 @@ LexdCompiler::buildPatternWithFlags(const pattern_element_t &tok, int pattern_st
         cerr << "FIXME" << endl;
       }
     }
+    if (verbose) cerr << "Done with " << to_ustring(printPattern(tok)) << endl;
     patternTransducers[tok] = trans;
   }
   else if(patternTransducers[tok] == NULL)
@@ -1455,7 +1690,7 @@ LexdCompiler::readFile(UFILE* infile)
 Transducer*
 LexdCompiler::buildTransducer(bool usingFlags)
 {
-  token_t start_tok = {.name = internName(" "), .part = 1};
+  token_t start_tok = {.name = internName(" "), .part = 1, .optional = false};
   pattern_element_t start_pat = {.left=start_tok, .right=start_tok,
                                  .tag_filter=tag_filter_t(),
                                  .mode=Normal};
@@ -1477,7 +1712,7 @@ Transducer*
 LexdCompiler::buildTransducerSingleLexicon()
 {
   tagsAsMinFlags = true;
-  token_t start_tok = {.name = internName(" "), .part = 1};
+  token_t start_tok = {.name = internName(" "), .part = 1, .optional = false};
   pattern_element_t start_pat = {.left=start_tok, .right=start_tok,
                                  .tag_filter=tag_filter_t(),
                                  .mode=Normal};
@@ -1486,10 +1721,12 @@ LexdCompiler::buildTransducerSingleLexicon()
   int end = buildPatternSingleLexicon(start_pat, 0);
   if(end == -1)
   {
-    die("No non-empty patterns found");
+    cerr << "WARNING: No non-empty patterns found." << endl;
+  }
+  else {
+    hyperminTrans->setFinal(end);
+    hyperminTrans->minimize();
   }
-  hyperminTrans->setFinal(end);
-  hyperminTrans->minimize();
   return hyperminTrans;
 }
 
@@ -1537,6 +1774,9 @@ LexdCompiler::insertEntry(Transducer* trans, const lex_seg_t &seg)
       state = trans->insertSingleTransduction((int)alphabet_lookup(flag, flag), state);
     }
   }
+  if (seg.regex != nullptr) {
+    state = trans->insertTransducer(state, *seg.regex);
+  }
   if(!shouldAlign)
   {
     for(unsigned int i = 0; i < seg.left.symbols.size() || i < seg.right.symbols.size(); i++)
@@ -1680,7 +1920,15 @@ LexdCompiler::getLexiconTransducer(pattern_element_t tok, unsigned int entry_ind
       continue;
     }
     Transducer* t = free ? trans[0] : new Transducer();
-    insertEntry(t, {.left=le.left, .right=re.right, .tags=tags});
+    if (le.regex != nullptr || re.regex != nullptr) {
+      if (tok.left.name.empty())
+        die("Cannot use %S one-sided - it contains a regex", err(name(tok.right.name)));
+      if (tok.right.name.empty())
+        die("Cannot use %S one-sided - it contains a regex", err(name(tok.left.name)));
+      if (tok.left.name != tok.right.name)
+        die("Cannot collate %S with %S - %S contains a regex", err(name(tok.left.name)), err(name(tok.right.name)), err(name((le.regex != nullptr ? tok.left.name : tok.right.name))));
+    }
+    insertEntry(t, {.left=le.left, .right=re.right, .regex=le.regex, .tags=tags});
     did_anything = true;
     if(!free)
     {
@@ -1688,6 +1936,17 @@ LexdCompiler::getLexiconTransducer(pattern_element_t tok, unsigned int entry_ind
       trans.push_back(t);
     }
   }
+  if(tok.optional()) {
+    Transducer* t = free ? trans[0] : new Transducer();
+    tags_t empty_tags;
+    insertEntry(t, {.left=empty.left, .right=empty.right, .regex=nullptr, .tags=empty_tags});
+    did_anything = true;
+    if (!free) {
+      applyMode(t, tok.mode);
+      trans.push_back(t);
+    }
+    did_anything = true;
+  }
   if(free)
   {
     if(!did_anything)
@@ -1789,6 +2048,15 @@ LexdCompiler::getLexiconTransducerWithFlags(pattern_element_t& tok, bool free)
     }
     did_anything = true;
     lex_seg_t seg;
+    if (le.regex != nullptr || re.regex != nullptr) {
+      if (tok.left.name.empty())
+        die("Cannot use %S one-sided - it contains a regex", err(name(tok.right.name)));
+      if (tok.right.name.empty())
+        die("Cannot use %S one-sided - it contains a regex", err(name(tok.left.name)));
+      if (tok.left.name != tok.right.name)
+        die("Cannot collate %S with %S - %S contains a regex", err(name(tok.left.name)), err(name(tok.right.name)), err(name((le.regex != nullptr ? tok.left.name : tok.right.name))));
+      seg.regex = le.regex;
+    }
     if(!free && tok.left.name.valid())
     {
       trans_sym_t flag = getFlag(Unification, tok.left.name, i);
@@ -1812,6 +2080,20 @@ LexdCompiler::getLexiconTransducerWithFlags(pattern_element_t& tok, bool free)
     seg.tags.insert(tags.begin(), tags.end());
     insertEntry(trans, seg);
   }
+  if(tok.optional()) {
+    lex_seg_t seg;
+    if (!free && tok.left.name.valid()) {
+      trans_sym_t flag = getFlag(Unification, tok.left.name, count);
+      seg.left.symbols.push_back(flag);
+      seg.right.symbols.push_back(flag);
+    }
+    if (!free && tok.right.name.valid() && tok.right.name != tok.left.name) {
+      trans_sym_t flag = getFlag(Unification, tok.right.name, count);
+      seg.left.symbols.push_back(flag);
+      seg.right.symbols.push_back(flag);
+    }
+    insertEntry(trans, seg);
+  }
   if(did_anything)
   {
     trans->minimize();
